#include "Clip.hpp"
#include "Matrix.hpp"

CClip::CClip(const CVertexFormat& c0, const CVertexFormat& c1, const CVertexFormat& c2,
	int w, int h)
{
	nVertex = 3;
	getVertex = 0;
	s[0] = c0;
	s[1] = c1;
	s[2] = c2;

	zMin = 0;
	zMax = zFar;
	xMin = 0;
	xMax = w - 1;
	yMin = 0;
	yMax = h - 1;

	nearClip();
	farClip();
	leftClip();
	rightClip();
	topClip();
	bottomClip();
}

void CClip::nearClip() {
	int cp = 0;
	int v0 = nVertex - 1;

	for (int v1 = 0; v1 < nVertex; v1++) {
		CVertexFormat* pV0 = &s[v0];
		CVertexFormat* pV1 = &s[v1];

		if (zMin <= pV0->z && zMin <= pV1->z) {
			d[cp] = *pV1;
			cp++;
		} else if (zMin <= pV0->z && pV1->z < zMin) {
			float t = (float)(zMin - pV0->z)
				/ (float)(pV1->z - pV0->z);
			d[cp].x = (int)(pV0->x + (pV1->x - pV0->x) * t);
			d[cp].y = (int)(pV0->y + (pV1->y - pV0->y) * t);
			d[cp].z = zMin;
			d[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			d[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			d[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			d[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			d[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
		} else if (pV0->z < zMin && zMin <= pV1->z) {
			float t = (float)(zMin - pV0->z)
				/ (float)(pV1->z - pV0->z);
			d[cp].x = (int)(pV0->x + (pV1->x - pV0->x) * t);
			d[cp].y = (int)(pV0->y + (pV1->y - pV0->y) * t);
			d[cp].z = zMin;
			d[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			d[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			d[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			d[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			d[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
			d[cp] = *pV1;
			cp++;
		}
		v0 = v1;
	}
	nVertex = cp;
}

void CClip::farClip() {
	int cp = 0;
	int v0 = nVertex - 1;

	for (int v1 = 0; v1 < nVertex; v1++) {
		CVertexFormat* pV0 = &d[v0];
		CVertexFormat* pV1 = &d[v1];

		if (pV0->z <= zMax && pV1->z <= zMax) {
			s[cp] = *pV1;
			cp++;
		}
		else if (pV0->z <= zMax && zMax < pV1->z) {
			float t = (float)(zMax - pV0->z)
				/ (float)(pV1->z - pV0->z);
			s[cp].x = (int)(pV0->x + (pV1->x - pV0->x) * t);
			s[cp].y = (int)(pV0->y + (pV1->y - pV0->y) * t);
			s[cp].z = zMax;
			s[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			s[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			s[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			s[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			s[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
		}
		else if (zMax < pV0->z && pV1->z <= zMax) {
			float t = (float)(zMax - pV0->z)
				/ (float)(pV1->z - pV0->z);
			s[cp].x = (int)(pV0->x + (pV1->x - pV0->x) * t);
			s[cp].y = (int)(pV0->y + (pV1->y - pV0->y) * t);
			s[cp].z = zMax;
			s[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			s[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			s[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			s[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			s[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
			s[cp] = *pV1;
			cp++;
		}
		v0 = v1;
	}
	nVertex = cp;
}

void CClip::leftClip() {
	int cp = 0;
	int v0 = nVertex - 1;

	for (int v1 = 0; v1 < nVertex; v1++) {
		CVertexFormat* pV0 = &s[v0];
		CVertexFormat* pV1 = &s[v1];

		if (xMin <= pV0->x && xMin <= pV1->x) {
			d[cp] = *pV1;
			cp++;
		} else if (xMin <= pV0->x && pV1->x < xMin) {
			float t = (float)(xMin - pV0->x)
				/ (float)(pV1->x - pV0->x);
			d[cp].x = xMin;
			d[cp].y = (int)(pV0->y + (pV1->y - pV0->y) * t);
			d[cp].z = (int)(pV0->z + (pV1->z - pV0->z) * t);
			d[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			d[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			d[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			d[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			d[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
		} else if (pV0->x < xMin && xMin <= pV1->x) {
			float t = (float)(xMin - pV0->x)
				/ (float)(pV1->x - pV0->x);
			d[cp].x = xMin;
			d[cp].y = (int)(pV0->y + (pV1->y - pV0->y) * t);
			d[cp].z = (int)(pV0->z + (pV1->z - pV0->z) * t);
			d[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			d[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			d[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			d[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			d[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
			d[cp] = *pV1;
			cp++;
		}
		v0 = v1;
	}
	nVertex = cp;
}

void CClip::rightClip() {
	int cp = 0;
	int v0 = nVertex - 1;

	for (int v1 = 0; v1 < nVertex; v1++) {
		CVertexFormat* pV0 = &d[v0];
		CVertexFormat* pV1 = &d[v1];

		if (pV0->x <= xMax && pV1->x <= xMax) {
			s[cp] = *pV1;
			cp++;
		} else if (pV0->x <= xMax && xMax < pV1->x) {
			float t = (float)(xMax - pV0->x)
				/ (float)(pV1->x - pV0->x);
			s[cp].x = xMax;
			s[cp].y = (int)(pV0->y + (pV1->y - pV0->y) * t);
			s[cp].z = (int)(pV0->z + (pV1->z - pV0->z) * t);
			s[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			s[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			s[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			s[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			s[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
		} else if (xMax < pV0->x && pV1->x <= xMax) {
			float t = (float)(xMax - pV0->x)
				/ (float)(pV1->x - pV0->x);
			s[cp].x = xMax;
			s[cp].y = (int)(pV0->y + (pV1->y - pV0->y) * t);
			s[cp].z = (int)(pV0->z + (pV1->z - pV0->z) * t);
			s[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			s[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			s[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			s[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			s[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
			s[cp] = *pV1;
			cp++;
		}
		v0 = v1;
	}
	nVertex = cp;
}

void CClip::topClip() {
	int cp = 0;
	int v0 = nVertex - 1;

	for (int v1 = 0; v1 < nVertex; v1++) {
		CVertexFormat* pV0 = &s[v0];
		CVertexFormat* pV1 = &s[v1];

		if (yMin <= pV0->y && yMin <= pV1->y) {
			d[cp] = *pV1;
			cp++;
		} else if (yMin <= pV0->y && pV1->y < yMin) {
			float t = (float)(yMin - pV0->y)
				/ (float)(pV1->y - pV0->y);
			d[cp].x = (int)(pV0->x + (pV1->x - pV0->x) * t);
			d[cp].y = yMin;
			d[cp].z = (int)(pV0->z + (pV1->z - pV0->z) * t);
			d[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			d[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			d[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			d[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			d[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
		} else if (pV0->y < yMin && yMin <= pV1->y) {
			float t = (float)(yMin - pV0->y)
				/ (float)(pV1->y - pV0->y);
			d[cp].x = (int)(pV0->x + (pV1->x - pV0->x) * t);
			d[cp].y = yMin;
			d[cp].z = (int)(pV0->z + (pV1->z - pV0->z) * t);
			d[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			d[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			d[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			d[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			d[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
			d[cp] = *pV1;
			cp++;
		}
		v0 = v1;
	}
	nVertex = cp;
}

void CClip::bottomClip() {
	int cp = 0;
	int v0 = nVertex - 1;

	for (int v1 = 0; v1 < nVertex; v1++) {
		CVertexFormat* pV0 = &d[v0];
		CVertexFormat* pV1 = &d[v1];

		if (pV0->y <= yMax && pV1->y <= yMax) {
			s[cp] = *pV1;
			cp++;
		} else if (pV0->y <= yMax && yMax < pV1->y) {
			float t = (float)(yMax - pV0->y)
				/ (float)(pV1->y - pV0->y);
			s[cp].x = (int)(pV0->x + (pV1->x - pV0->x) * t);
			s[cp].y = yMax;
			s[cp].z = (int)(pV0->z + (pV1->z - pV0->z) * t);
			s[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			s[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			s[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			s[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			s[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
		} else if (yMax < pV0->y && pV1->y <= yMax) {
			float t = (float)(yMax - pV0->y)
				/ (float)(pV1->y - pV0->y);
			s[cp].x = (int)(pV0->x + (pV1->x - pV0->x) * t);
			s[cp].y = yMax;
			s[cp].z = (int)(pV0->z + (pV1->z - pV0->z) * t);
			s[cp].rhw = (int)(pV0->rhw + (pV1->rhw - pV0->rhw) * t);
			s[cp].u = (int)(pV0->u + (pV1->u - pV0->u) * t);
			s[cp].v = (int)(pV0->v + (pV1->v - pV0->v) * t);
			s[cp].bright = (int)(pV0->bright + (pV1->bright - pV0->bright) * t);
			s[cp].specular = (int)(pV0->specular + (pV1->specular - pV0->specular) * t);
			cp++;
			s[cp] = *pV1;
			cp++;
		}
		v0 = v1;
	}
	nVertex = cp;
}

bool CClip::getTriangle(CVertexFormat* pC0, CVertexFormat* pC1, CVertexFormat* pC2) {
	if (nVertex < 3) {
		return false;
	}

	if (nVertex < 3 + getVertex) {
		return false;
	}

	*pC0 = s[0];
	*pC1 = s[1 + getVertex];
	*pC2 = s[2 + getVertex];
	getVertex++;
	return true;
}
