#ifndef Coordinate_h
#define Coordinate_h

#include "Porting.hpp"

#include <math.h>

struct VERTEX {
	float x, y, z;
	float nx, ny, nz;
	float u, v;
};

class CVector {
public:
	CVector() {}

public:
	float x, y, z;

public:
	CVector(float x, float y, float z) {
		this->x = x;
		this->y = y;
		this->z = z;
	}

public:
	void normalize() {
		float l = sqrtf(x * x + y * y + z * z);
		if (l) {
			x /= l;
			y /= l;
			z /= l;
		}
		else {
			x = 0.0f;
			y = 0.0f;
			z = 0.0f;
		}
	}
	static CVector cross(const CVector left, const CVector right) {
		CVector r;
		r.x = left.y * right.z - left.z * right.y;
		r.y = left.z * right.x - left.x * right.z;
		r.z = left.x * right.y - left.y * right.x;
		return r;
	}
	static float dot(const CVector left, const CVector right) {
		return left.x * right.x
			+ left.y * right.y + left.z * right.z;
	}
	CVector operator + (CVector& dst) const {
		CVector v(x + dst.x, y + dst.y, z + dst.z);
		return v;
	}
	CVector operator - (CVector& dst) const {
		CVector v(x - dst.x, y - dst.y, z - dst.z);
		return v;
	}
};

class CVertexFormat
{
public:
	CVertexFormat() {}

public:
	int x;
	int y;
	int z;
	int rhw;
	int u;
	int v;
	int bright;
	int specular;

public:
	CVertexFormat(int x, int y, int z, int rhw, int u, int v, int bright, int specular) {
		this->x = x;
		this->y = y;
		this->z = z;
		this->rhw = rhw;
		this->u = u;
		this->v = v;
		this->bright = bright;
		this->specular = specular;
	}
};

#endif /* Coordinate_h */
