#ifndef _Frame_hpp_
#define _Frame_hpp_

#include "Porting.hpp"

#include "Coordinate.hpp"
#include "Matrix.hpp"
#include "Texture.hpp"

class CFrame
{
private:
	CFrame();	// 引数無しコンストラクタは使わせない

public:
	CFrame(int w, int h, DWORD* pBits);
	virtual ~CFrame();

private:
	int w;
	int h;
	DWORD* pBits;
	USHORT* pZbuffer;

	CMatrix world;
	CMatrix view;
	CMatrix projection;
	CMatrix viewport;
	CMatrix affine;

	CVector eye;

	CVector light;
	float ambient;

public:
	void setWorld(const CMatrix& m) { world = m; }
	void setView(const CMatrix& m, const CVector& potision) {
		view = m;
		eye = potision;
		eye.normalize();
	}
	void setProjection(const CMatrix& m) { projection = m; }
	void setViewport(const CMatrix& m) { viewport = m; }

	void setLight(const CVector& light, float ambient) {
		this->light = light;
		this->ambient = ambient;
	}

private:
	void upperTriangle(CTexture* pTex,
		CVertexFormat& c0, CVertexFormat& c1, CVertexFormat& c2);
	void lowerTriangle(CTexture* pTex,
		CVertexFormat& c0, CVertexFormat& c1, CVertexFormat& c2);
	void leftTriangle(CTexture* pTex,
		CVertexFormat& c0, CVertexFormat& c1, CVertexFormat& c2);
	void rightTriangle(CTexture* pTex,
		CVertexFormat& c0, CVertexFormat& c1, CVertexFormat& c2);

	void triangle(CTexture* pTex,
		CVertexFormat& c0, CVertexFormat& c1, CVertexFormat& c2);

	int lighting(const VERTEX& v, int* pSpecular);

public:
	int W() { return w; }
	int H() { return h; }

	void clear();

	void transform();

	void triangle(CTexture* pTex, const VERTEX& p0, const VERTEX& p1, const VERTEX& p2);
};

template <class T>
T clamp(T x, T min, T max) {
	if (x < min) {
		return min;
	}
	if (max < x) {
		return max;
	}
	return x;
}

#endif /* _Frame_hpp_ */
