#include "Frame.hpp"
#include "Scan.hpp"
#include "Matrix.hpp"
#include "Clip.hpp"

CFrame::CFrame(int w, int h, DWORD* pBits) {
	this->w = w;
	this->h = h;
	this->pBits = pBits;

	pZbuffer = new USHORT[w * h];
}

CFrame::~CFrame()
{
	delete[] pZbuffer;
	pZbuffer = NULL;
}

void CFrame::clear() {
	memset(pBits, 0, w * h * sizeof(DWORD));
	memset(pZbuffer, 0xff, w * h * sizeof(USHORT));
}

inline DWORD A(DWORD color) { return (color & 0xff000000) >> 24; }
inline DWORD R(DWORD color) { return color & 0x00ff0000; }
inline DWORD G(DWORD color) { return color & 0x0000ff00; }
inline DWORD B(DWORD color) { return color & 0x000000ff; }
inline DWORD XRGB(DWORD r, DWORD g, DWORD b) {
	return (r & 0x00ff0000) | (g & 0x0000ff00) | (b & 0x000000ff);
}
inline void plot(int x, int y, int z,
	int u, int v, DWORD* pTexture, int texW,
	DWORD* pBits, USHORT* pZbuffer, int w,
	int bright, int specular) {
	if (z < pZbuffer[y * w + x]) {
		DWORD color = pTexture[v * texW + u];
		DWORD a = A(color);
		if (a == 0) {
			return;		// 透明なので描かない
		}
		pZbuffer[y * w + x] = z;
		DWORD r = R(color);
		r = (r * bright) >> 8;		// * (0〜255) / 256
		r += (DWORD)specular << 16;
		if (((DWORD)255 << 16) < r) {
			r = (DWORD)255 << 16;
		}
		DWORD g = G(color);
		g = (g * bright) >> 8;		// * (0〜255) / 256
		g += (DWORD)specular << 8;
		if (((DWORD)255 << 8) < g) {
			g = (DWORD)255 << 8;
		}
		DWORD b = B(color);
		b = (b * bright) >> 8;		// * (0〜255) / 256
		b += (DWORD)specular;
		if ((DWORD)255 < b) {
			b = (DWORD)255;
		}
		if (a < 255) {	// 半透明なのでαブレンディング
			DWORD colorD = pBits[y * w + x];
			DWORD rD = R(colorD);
			r = ((r - rD) * a >> 8) + rD;	// 本来は >> 8 ではなく / 255
			DWORD gD = G(colorD);
			g = ((g - gD) * a >> 8) + gD;	// 本来は >> 8 ではなく / 255
			DWORD bD = B(colorD);
			b = ((b - bD) * a >> 8) + bD;	// 本来は >> 8 ではなく / 255
		}
		pBits[y * w + x] = XRGB(r, g, b);
	}
}
void CFrame::upperTriangle(CTexture* pTex,
	CVertexFormat& c0, CVertexFormat& c1, CVertexFormat& c2) {
	DWORD* pTexture = pTex->pBits();
	int texW = pTex->W();
	int texH = pTex->H();

	CScan p1(c0.y, c1.y, c0.x, c1.x);
	CScan p2(c0.y, c2.y, c0.x, c2.x);
	CScan z1(c0.y, c1.y, c0.z, c1.z);
	CScan z2(c0.y, c2.y, c0.z, c2.z);
	CScan w1(c0.y, c1.y, c0.rhw, c1.rhw);
	CScan w2(c0.y, c1.y, c0.rhw, c2.rhw);
	CScan u1(c0.y, c1.y, c0.u, c1.u);
	CScan u2(c0.y, c2.y, c0.u, c2.u);
	CScan v1(c0.y, c1.y, c0.v, c1.v);
	CScan v2(c0.y, c2.y, c0.v, c2.v);
	CScan b1(c0.y, c1.y, c0.bright, c1.bright);
	CScan b2(c0.y, c2.y, c0.bright, c2.bright);
	CScan s1(c0.y, c1.y, c0.specular, c1.specular);
	CScan s2(c0.y, c2.y, c0.specular, c2.specular);

	int y, sx, ex;
	int sz, ez;
	int sw, ew;
	int su, eu;
	int sv, ev;
	int sb, eb;
	int ss, es;
	while (p1.linear(&y, &sx)) {
		p2.linear(&y, &ex);
		z1.linear(&y, &sz);
		z2.linear(&y, &ez);
		w1.linear(&y, &sw);
		w2.linear(&y, &ew);
		u1.linear(&y, &su);
		u2.linear(&y, &eu);
		v1.linear(&y, &sv);
		v2.linear(&y, &ev);
		b1.linear(&y, &sb);
		b2.linear(&y, &eb);
		s1.linear(&y, &ss);
		s2.linear(&y, &es);

		CScan lineZ(sx, ex, sz, ez);
		CScan lineW(sx, ex, sw, ew);
		CScan lineU(sx, ex, su, eu);
		CScan lineV(sx, ex, sv, ev);
		CScan lineB(sx, ex, sb, eb);
		CScan lineS(sx, ex, ss, es);
		int x, z, rhw, u, v, bright, specular;
		while (lineZ.linear(&x, &z)) {
			lineW.linear(&x, &rhw);
			lineU.linear(&x, &u);
			lineV.linear(&x, &v);
			lineB.linear(&x, &bright);
			lineS.linear(&x, &specular);
			if (rhw == 0) {
				continue;	// 0割ガード
			}
			u /= rhw;
			if (texW <= u) { u = texW - 1; }
			v /= rhw;
			if (texH <= v) { v = texH - 1; }
			plot(x, y, z, u, v, pTexture, texW, pBits, pZbuffer, W(), bright, specular);
		}
	}
}

void CFrame::lowerTriangle(CTexture* pTex,
	CVertexFormat& c0, CVertexFormat& c1, CVertexFormat& c2) {
	DWORD* pTexture = pTex->pBits();
	int texW = pTex->W();
	int texH = pTex->H();

	CScan p0(c0.y, c1.y, c0.x, c1.x);
	CScan p2(c2.y, c1.y, c2.x, c1.x);
	CScan z0(c0.y, c1.y, c0.z, c1.z);
	CScan z2(c2.y, c1.y, c2.z, c1.z);
	CScan w0(c0.y, c1.y, c0.rhw, c1.rhw);
	CScan w2(c2.y, c1.y, c2.rhw, c1.rhw);
	CScan u0(c0.y, c1.y, c0.u, c1.u);
	CScan u2(c2.y, c1.y, c2.u, c1.u);
	CScan v0(c0.y, c1.y, c0.v, c1.v);
	CScan v2(c2.y, c1.y, c2.v, c1.v);
	CScan b0(c0.y, c1.y, c0.bright, c1.bright);
	CScan b2(c2.y, c1.y, c2.bright, c1.bright);
	CScan s0(c0.y, c1.y, c0.specular, c1.specular);
	CScan s2(c2.y, c1.y, c2.specular, c1.specular);

	int y, sx, ex;
	int sz, ez;
	int sw, ew;
	int su, eu;
	int sv, ev;
	int sb, eb;
	int ss, es;
	while (p0.linear(&y, &sx)) {
		p2.linear(&y, &ex);
		z0.linear(&y, &sz);
		z2.linear(&y, &ez);
		w0.linear(&y, &sw);
		w2.linear(&y, &ew);
		u0.linear(&y, &su);
		u2.linear(&y, &eu);
		v0.linear(&y, &sv);
		v2.linear(&y, &ev);
		b0.linear(&y, &sb);
		b2.linear(&y, &eb);
		s0.linear(&y, &ss);
		s2.linear(&y, &es);
		CScan lineZ(sx, ex, sz, ez);
		CScan lineW(sx, ex, sw, ew);
		CScan lineU(sx, ex, su, eu);
		CScan lineV(sx, ex, sv, ev);
		CScan lineB(sx, ex, sb, eb);
		CScan lineS(sx, ex, ss, es);
		int x, z, rhw, u, v, bright, specular;
		while (lineZ.linear(&x, &z)) {
			lineW.linear(&x, &rhw);
			lineU.linear(&x, &u);
			lineV.linear(&x, &v);
			lineB.linear(&x, &bright);
			lineS.linear(&x, &specular);
			if (rhw == 0) {
				continue;	// 0割ガード
			}
			u /= rhw;
			if (texW <= u) { u = texW - 1; }
			v /= rhw;
			if (texH <= v) { v = texH - 1; }
			plot(x, y, z, u, v, pTexture, texW, pBits, pZbuffer, W(), bright, specular);
		}
	}
}

void CFrame::leftTriangle(CTexture* pTex,
	CVertexFormat& c0, CVertexFormat& c1, CVertexFormat& c2) {
	DWORD* pTexture = pTex->pBits();
	int texW = pTex->W();
	int texH = pTex->H();

	CScan p1(c0.y, c1.y, c0.x, c1.x);
	CScan p2(c0.y, c2.y, c0.x, c2.x);
	CScan z1(c0.y, c1.y, c0.z, c1.z);
	CScan z2(c0.y, c2.y, c0.z, c2.z);
	CScan w1(c0.y, c1.y, c0.rhw, c1.rhw);
	CScan w2(c0.y, c2.y, c0.rhw, c2.rhw);
	CScan u1(c0.y, c1.y, c0.u, c1.u);
	CScan u2(c0.y, c2.y, c0.u, c2.u);
	CScan v1(c0.y, c1.y, c0.v, c1.v);
	CScan v2(c0.y, c2.y, c0.v, c2.v);
	CScan b1(c0.y, c1.y, c0.bright, c1.bright);
	CScan b2(c0.y, c2.y, c0.bright, c2.bright);
	CScan s1(c0.y, c1.y, c0.specular, c1.specular);
	CScan s2(c0.y, c2.y, c0.specular, c2.specular);

	int y, sx, ex;
	int sz, ez;
	int sw, ew;
	int su, eu;
	int sv, ev;
	int sb, eb;
	int ss, es;
	while (p1.linear(&y, &sx)) {
		p2.linear(&y, &ex);
		z1.linear(&y, &sz);
		z2.linear(&y, &ez);
		w1.linear(&y, &sw);
		w2.linear(&y, &ew);
		u1.linear(&y, &su);
		u2.linear(&y, &eu);
		v1.linear(&y, &sv);
		v2.linear(&y, &ev);
		b1.linear(&y, &sb);
		b2.linear(&y, &eb);
		s1.linear(&y, &ss);
		s2.linear(&y, &es);

		CScan lineZ(sx, ex, sz, ez);
		CScan lineW(sx, ex, sw, ew);
		CScan lineU(sx, ex, su, eu);
		CScan lineV(sx, ex, sv, ev);
		CScan lineB(sx, ex, sb, eb);
		CScan lineS(sx, ex, ss, es);
		int x, z, rhw, u, v, bright, specular;
		while (lineZ.linear(&x, &z)) {
			lineW.linear(&x, &rhw);
			lineU.linear(&x, &u);
			lineV.linear(&x, &v);
			lineB.linear(&x, &bright);
			lineS.linear(&x, &specular);
			if (rhw == 0) {
				continue;	// 0割ガード
			}
			u /= rhw;
			if (texW <= u) { u = texW - 1; }
			v /= rhw;
			if (texH <= v) { v = texH - 1; }
			plot(x, y, z, u, v, pTexture, texW, pBits, pZbuffer, W(), bright, specular);
		}
	}

	CScan p1n(c1.y, c2.y, c1.x, c2.x);
	CScan z1n(c1.y, c2.y, c1.z, c2.z);
	CScan w1n(c1.y, c2.y, c1.rhw, c2.rhw);
	CScan u1n(c1.y, c2.y, c1.u, c2.u);
	CScan v1n(c1.y, c2.y, c1.v, c2.v);
	CScan b1n(c1.y, c2.y, c1.bright, c2.bright);
	CScan s1n(c1.y, c2.y, c1.specular, c2.specular);

	while (p1n.linear(&y, &sx)) {
		p2.linear(&y, &ex);
		z1n.linear(&y, &sz);
		z2.linear(&y, &ez);
		w1n.linear(&y, &sw);
		w2.linear(&y, &ew);
		u1n.linear(&y, &su);
		u2.linear(&y, &eu);
		v1n.linear(&y, &sv);
		v2.linear(&y, &ev);
		b1n.linear(&y, &sb);
		b2.linear(&y, &eb);
		s1n.linear(&y, &ss);
		s2.linear(&y, &es);

		CScan lineZ(sx, ex, sz, ez);
		CScan lineW(sx, ex, sw, ew);
		CScan lineU(sx, ex, su, eu);
		CScan lineV(sx, ex, sv, ev);
		CScan lineB(sx, ex, sb, eb);
		CScan lineS(sx, ex, ss, es);
		int x, z, rhw, u, v, bright, specular;
		while (lineZ.linear(&x, &z)) {
			lineW.linear(&x, &rhw);
			lineU.linear(&x, &u);
			lineV.linear(&x, &v);
			lineB.linear(&x, &bright);
			lineS.linear(&x, &specular);
			if (rhw == 0) {
				continue;	// 0割ガード
			}
			u /= rhw;
			if (texW <= u) { u = texW - 1; }
			v /= rhw;
			if (texH <= v) { v = texH - 1; }
			plot(x, y, z, u, v, pTexture, texW, pBits, pZbuffer, W(), bright, specular);
		}
	}
}

void CFrame::rightTriangle(CTexture* pTex,
	CVertexFormat& c0, CVertexFormat& c1, CVertexFormat& c2) {
	DWORD* pTexture = pTex->pBits();
	int texW = pTex->W();
	int texH = pTex->H();

	CScan p1(c0.y, c1.y, c0.x, c1.x);
	CScan p2(c0.y, c2.y, c0.x, c2.x);
	CScan z1(c0.y, c1.y, c0.z, c1.z);
	CScan z2(c0.y, c2.y, c0.z, c2.z);
	CScan w1(c0.y, c1.y, c0.rhw, c1.rhw);
	CScan w2(c0.y, c2.y, c0.rhw, c2.rhw);
	CScan u1(c0.y, c1.y, c0.u, c1.u);
	CScan u2(c0.y, c2.y, c0.u, c2.u);
	CScan v1(c0.y, c1.y, c0.v, c1.v);
	CScan v2(c0.y, c2.y, c0.v, c2.v);
	CScan b1(c0.y, c1.y, c0.bright, c1.bright);
	CScan b2(c0.y, c2.y, c0.bright, c2.bright);
	CScan s1(c0.y, c1.y, c0.specular, c1.specular);
	CScan s2(c0.y, c2.y, c0.specular, c2.specular);

	int y, sx, ex;
	int sz, ez;
	int sw, ew;
	int su, eu;
	int sv, ev;
	int sb, eb;
	int ss, es;
	while (p2.linear(&y, &ex)) {
		p1.linear(&y, &sx);
		z2.linear(&y, &ez);
		z1.linear(&y, &sz);
		w2.linear(&y, &ew);
		w1.linear(&y, &sw);
		u2.linear(&y, &eu);
		u1.linear(&y, &su);
		v2.linear(&y, &ev);
		v1.linear(&y, &sv);
		b2.linear(&y, &eb);
		b1.linear(&y, &sb);
		s2.linear(&y, &es);
		s1.linear(&y, &ss);

		CScan lineZ(sx, ex, sz, ez);
		CScan lineW(sx, ex, sw, ew);
		CScan lineU(sx, ex, su, eu);
		CScan lineV(sx, ex, sv, ev);
		CScan lineB(sx, ex, sb, eb);
		CScan lineS(sx, ex, ss, es);
		int x, z, rhw, u, v, bright, specular;
		while (lineZ.linear(&x, &z)) {
			lineW.linear(&x, &rhw);
			lineU.linear(&x, &u);
			lineV.linear(&x, &v);
			lineB.linear(&x, &bright);
			lineS.linear(&x, &specular);
			if (rhw == 0) {
				continue;	// 0割ガード
			}
			u /= rhw;
			if (texW <= u) { u = texW - 1; }
			v /= rhw;
			if (texH <= v) { v = texH - 1; }
			plot(x, y, z, u, v, pTexture, texW, pBits, pZbuffer, W(), bright, specular);
		}
	}

	CScan p2n(c2.y, c1.y, c2.x, c1.x);
	CScan z2n(c2.y, c1.y, c2.z, c1.z);
	CScan w2n(c2.y, c1.y, c2.rhw, c1.rhw);
	CScan u2n(c2.y, c1.y, c2.u, c1.u);
	CScan v2n(c2.y, c1.y, c2.v, c1.v);
	CScan b2n(c2.y, c1.y, c2.bright, c1.bright);
	CScan s2n(c2.y, c1.y, c2.specular, c1.specular);

	while (p2n.linear(&y, &ex)) {
		p1.linear(&y, &sx);
		z2n.linear(&y, &ez);
		z1.linear(&y, &sz);
		w2n.linear(&y, &ew);
		w1.linear(&y, &sw);
		u2n.linear(&y, &eu);
		u1.linear(&y, &su);
		v2n.linear(&y, &ev);
		v1.linear(&y, &sv);
		b2n.linear(&y, &eb);
		b1.linear(&y, &sb);
		s2n.linear(&y, &es);
		s1.linear(&y, &ss);

		CScan lineZ(sx, ex, sz, ez);
		CScan lineW(sx, ex, sw, ew);
		CScan lineU(sx, ex, su, eu);
		CScan lineV(sx, ex, sv, ev);
		CScan lineB(sx, ex, sb, eb);
		CScan lineS(sx, ex, ss, es);
		int x, z, rhw, u, v, bright, specular;
		while (lineZ.linear(&x, &z)) {
			lineW.linear(&x, &rhw);
			lineU.linear(&x, &u);
			lineV.linear(&x, &v);
			lineB.linear(&x, &bright);
			lineS.linear(&x, &specular);
			if (rhw == 0) {
				continue;	// 0割ガード
			}
			u /= rhw;
			if (texW <= u) { u = texW - 1; }
			v /= rhw;
			if (texH <= v) { v = texH - 1; }
			plot(x, y, z, u, v, pTexture, texW, pBits, pZbuffer, W(), bright, specular);
		}
	}
}

void CFrame::triangle(CTexture* pTex,
	CVertexFormat& c0, CVertexFormat& c1, CVertexFormat& c2) {
	if (c0.y < c1.y && c0.y < c2.y) {
		if (c1.y == c2.y) {
			upperTriangle(pTex, c0, c1, c2);
			return;
		}

		if (c1.y < c2.y) {
			leftTriangle(pTex, c0, c1, c2);
			return;
		}

		rightTriangle(pTex, c0, c1, c2);
		return;
	}

	if (c1.y < c2.y && c1.y < c0.y) {
		if (c2.y == c0.y) {
			upperTriangle(pTex, c1, c2, c0);
			return;
		}

		if (c2.y < c0.y) {
			leftTriangle(pTex, c1, c2, c0);
			return;
		}

		rightTriangle(pTex, c1, c2, c0);
		return;
	}

	if (c2.y < c0.y && c2.y < c1.y) {
		if (c0.y == c1.y) {
			upperTriangle(pTex, c2, c0, c1);
			return;
		}

		if (c0.y < c1.y) {
			leftTriangle(pTex, c2, c0, c1);
			return;
		}

		rightTriangle(pTex, c2, c0, c1);
		return;
	}

	if (c0.y == c2.y) {
		lowerTriangle(pTex, c0, c1, c2);
		return;
	}

	if (c1.y == c0.y) {
		lowerTriangle(pTex, c1, c2, c0);
		return;
	}

	if (c2.y == c1.y) {
		lowerTriangle(pTex, c2, c0, c1);
		return;
	}

	return;
}

void CFrame::transform() {
	affine.identity();
	affine.multiply(world);
	affine.multiply(view);
	affine.multiply(projection);
	affine.multiply(viewport);
}

int CFrame::lighting(const VERTEX& v, int* pSpecular) {
	CVector p = world.multiply(v.x, v.y, v.z);
	CVector pl = light - p;
	pl.normalize();

	CVector n = world.multiply(v.x + v.nx, v.y + v.ny, v.z + v.nz);
	n = n - p;
	n.normalize();
	float diffuse = clamp(CVector::dot(pl, n), 0.0f, 1.0f);
	float bright = clamp(diffuse + ambient, 0.0f, 1.0f);

	CVector pe = eye - p;
	pe.normalize();
	CVector H = pl + pe;
	H.normalize();
	float specular = CVector::dot(n, H);
	specular = clamp(powf(specular, 5), 0.0f, 1.0f);
	*pSpecular = (int)(specular * 255.0f);	// 255.0fは0.0f〜1.0fを0〜255に展開している
	return (int)(bright * 255.0f);	// 255.0fは0.0f〜1.0fを0〜255に展開している
}

void CFrame::triangle(CTexture* pTex, const VERTEX& p0, const VERTEX& p1, const VERTEX& p2) {
	int x0, y0, z0, rhw0;
	if (!affine.transform(&x0, &y0, &z0, &rhw0, p0)) {
		return;
	}
	int u0 = (int)((pTex->W() - 1) * p0.u) * rhw0;
	int v0 = (int)((pTex->H() - 1) * p0.v) * rhw0;
	int specular0;
	int bright0 = lighting(p0, &specular0);
	CVertexFormat c0(x0, y0, z0, rhw0, u0, v0, bright0, specular0);
	int x1, y1, z1, rhw1;
	if (!affine.transform(&x1, &y1, &z1, &rhw1, p1)) {
		return;
	}
	int u1 = (int)((pTex->W() - 1) * p1.u) * rhw1;
	int v1 = (int)((pTex->H() - 1) * p1.v) * rhw1;
	int specular1;
	int bright1 = lighting(p1, &specular1);
	CVertexFormat c1(x1, y1, z1, rhw1, u1, v1, bright1, specular1);
	int x2, y2, z2, rhw2;
	if (!affine.transform(&x2, &y2, &z2, &rhw2, p2)) {
		return;
	}
	int u2 = (int)((pTex->W() - 1) * p2.u) * rhw2;
	int v2 = (int)((pTex->H() - 1) * p2.v) * rhw2;
	int specular2;
	int bright2 = lighting(p2, &specular2);
	CVertexFormat c2(x2, y2, z2, rhw2, u2, v2, bright2, specular2);

	int z = (c1.x - c0.x) * (c2.y - c0.y)
		- (c1.y - c0.y) * (c2.x - c0.x);
	if (0 <= z) {
		return;	// 背面カリング
	}

	CClip clip(c0, c1, c2, W(), H());
	while (clip.getTriangle(&c0, &c1, &c2)) {
		triangle(pTex, c0, c1, c2);
	}
}
