#include "Frame.hpp"
#include "Texture.hpp"
#include "Matrix.hpp"
#include "TestApp.hpp"

CApp* appStartup(CFrame* pFrame) {
	return new CTestApp(pFrame);
}

CTestApp::CTestApp(CFrame* pFrame)
{
	this->pFrame = pFrame;
	this->pTexture0 = new CTexture("saikoro/1.png");
	this->pTexture1 = new CTexture("saikoro/6.png");
	this->pTexture2 = new CTexture("saikoro/3.png");
	this->pTexture3 = new CTexture("saikoro/4.png");
	this->pTexture4 = new CTexture("saikoro/5.png");
	this->pTexture5 = new CTexture("saikoro/2.png");
	angle = 0.0f;

	this->pLight = new CTexture("light/light.png");
	time = 0.0f;

	CMatrix m;
	m.projection(0.5f, (float)pFrame->W() / (float)pFrame->H(),
		1.0f, 100.0f);
	pFrame->setProjection(m);

	m.viewport(pFrame->W(), pFrame->H());
	pFrame->setViewport(m);

}

CTestApp::~CTestApp()
{
	delete pTexture0;
	delete pTexture1;
	delete pTexture2;	
	delete pTexture3;
	delete pTexture4;
	delete pTexture5;

	delete pLight;
}

bool CTestApp::appDraw() {
	// 頂点反時計回りがおもて
	//1
	const VERTEX p0[] = {
		{ 1.0f,-1.0f,-1.0f, 0.0f,0.0f,-1.0f, 0.0f,0.0f },
		{ 1.0f,1.0f,-1.0f, 0.0f,0.0f,-1.0f, 0.0f,1.0f },
		{ -1.0f,1.0f,-1.0f, 0.0f,0.0f,-1.0f, 1.0f,1.0f } };
	const VERTEX p1[] = {
		{ 1.0f,-1.0f,-1.0f, 0.0f,0.0f,-1.0f, 0.0f,0.0f },
		{ -1.0f,1.0f,-1.0f, 0.0f,0.0f,-1.0f, 1.0f,1.0f },
		{ -1.0f,-1.0f,-1.0f, 0.0f,0.0f,-1.0f, 1.0f,0.0f } };

	//6
	const VERTEX p2[] = {
		{ 1.0f,-1.0f,1.0f, 0.0f,0.0f,1.0f, 0.0f,0.0f },
		{ -1.0f,1.0f,1.0f, 0.0f,0.0f,1.0f, 1.0f,1.0f },
		{ 1.0f,1.0f,1.0f, 0.0f,0.0f,1.0f, 0.0f,1.0f }  };
	const VERTEX p3[] = {
		{ 1.0f,-1.0f,1.0f, 0.0f,0.0f,1.0f, 0.0f,0.0f },
		{ -1.0f,-1.0f,1.0f, 0.0f,0.0f,1.0f, 1.0f,0.0f },
		{ -1.0f,1.0f,1.0f, 0.0f,0.0f,1.0f, 1.0f,1.0f }  };

	//3
	const VERTEX p4[] = {
		{ 1.0f,1.0f,1.0f, 1.0f,0.0f,0.0f, 0.0f,0.0f },
		{ 1.0f,1.0f,-1.0f, 1.0f,0.0f,0.0f, 0.0f,1.0f },
		{ 1.0f,-1.0f,-1.0f, 1.0f,0.0f,0.0f, 1.0f,1.0f }  };
	const VERTEX p5[] = {
		{ 1.0f,1.0f,1.0f, 1.0f,0.0f,0.0f, 0.0f,0.0f },
		{ 1.0f,-1.0f,-1.0f, 1.0f,0.0f,0.0f, 1.0f,1.0f },
		{ 1.0f,-1.0f,1.0f, 1.0f,0.0f,0.0f, 1.0f,0.0f }  };
	
	//4
	const VERTEX p6[] = {
		{ -1.0f,1.0f,-1.0f, -1.0f,0.0f,0.0f, 0.0f,0.0f },
		{ -1.0f,-1.0f,1.0f, -1.0f,0.0f,0.0f, 1.0f,1.0f },
		{ -1.0f,-1.0f,-1.0f, -1.0f,0.0f,0.0f, 0.0f,1.0f }  };
	const VERTEX p7[] = {
		{ -1.0f,1.0f,-1.0f, -1.0f,0.0f,0.0f, 0.0f,0.0f },
		{ -1.0f,1.0f,1.0f, -1.0f,0.0f,0.0f, 1.0f,0.0f },
		{ -1.0f,-1.0f,1.0f, -1.0f,0.0f,0.0f, 1.0f,1.0f }  };

	//5
	const VERTEX p8[] = {
		{ -1.0f,-1.0f,1.0f, 0.0f,-1.0f,0.0f, 0.0f,0.0f },
		{ 1.0f,-1.0f,-1.0f, 0.0f,-1.0f,0.0f, 1.0f,1.0f },
		{ -1.0f,-1.0f,-1.0f, 0.0f,-1.0f,0.0f, 0.0f,1.0f }  };
	const VERTEX p9[] = {
		{ -1.0f,-1.0f,1.0f, 0.0f,-1.0f,0.0f, 0.0f,0.0f },
		{ 1.0f,-1.0f,1.0f, 0.0f,-1.0f,0.0f, 1.0f,0.0f },
		{ 1.0f,-1.0f,-1.0f, 0.0f,-1.0f,0.0f, 1.0f,1.0f }  };

	//2
	const VERTEX p10[] = {
		{ -1.0f,1.0f,1.0f, 0.0f,1.0f,0.0f, 0.0f,0.0f },
		{ 1.0f,1.0f,-1.0f, 0.0f,1.0f,0.0f, 1.0f,1.0f },
		{ 1.0f,1.0f,1.0f, 0.0f,1.0f,0.0f, 0.0f,1.0f } };
	const VERTEX p11[] = {
		{ -1.0f,1.0f,1.0f, 0.0f,1.0f,0.0f, 0.0f,0.0f },
		{ -1.0f,1.0f,-1.0f, 0.0f,1.0f,0.0f, 1.0f,0.0f },
		{ 1.0f,1.0f,-1.0f, 0.0f,1.0f,0.0f, 1.0f,1.0f } };
	
	// 点光源
	const VERTEX light0[] = {
		{ 1.0f,-1.0f,0.0f, 0.0f,0.0f,0.0f, 0.0f,0.0f },
		{ 1.0f,1.0f,0.0f, 0.0f,0.0f,0.0f, 0.0f,1.0f },
		{ -1.0f,1.0f,0.0f, 0.0f,0.0f,0.0f, 1.0f,1.0f } };
	const VERTEX light1[] = {
		{ 1.0f,-1.0f,0.0f, 0.0f,0.0f,0.0f, 0.0f,0.0f },
		{ -1.0f,1.0f,0.0f, 0.0f,0.0f,0.0f, 1.0f,1.0f },
		{ -1.0f,-1.0f,0.0f, 0.0f,0.0f,0.0f, 1.0f,0.0f } };

	CMatrix m;
	m.scale(1.0f);
	m.rotateZ(2.0f * (float)M_PI * angle);
	m.rotateY(2.0f * (float)M_PI * angle);
	m.translate(0.0f, 0.0f, 0.0f);
	pFrame->setWorld(m);
	angle += 0.002f;
	if (1.0f < angle) {
		angle = 0.0f;
	}

	CVector position(0.0f, 0.0f, -3.0f);
	CVector target(0.0f, 0.0f, 0.0f);
	CVector up(0.0f, 1.0f, 0.0f);
	m.lookAt(position, target, up);
	pFrame->setView(m, position);

	float d = 2.0f;
	CVector light(d * cosf(time), d * sinf(time * 0.8f), d * sinf(time));
	pFrame->setLight(light, 0.5f);

	pFrame->transform();

	//1
	pFrame->triangle(pTexture0, p0[0], p0[1], p0[2]);
	pFrame->triangle(pTexture0, p1[0], p1[1], p1[2]);

	//6
	pFrame->triangle(pTexture1, p2[0], p2[1], p2[2]);
	pFrame->triangle(pTexture1, p3[0], p3[1], p3[2]);

	//3
	pFrame->triangle(pTexture2, p4[0], p4[1], p4[2]);
	pFrame->triangle(pTexture2, p5[0], p5[1], p5[2]);
	
	//4
	pFrame->triangle(pTexture3, p6[0], p6[1], p6[2]);
	pFrame->triangle(pTexture3, p7[0], p7[1], p7[2]);

	//5
	pFrame->triangle(pTexture4, p8[0], p8[1], p8[2]);
	pFrame->triangle(pTexture4, p9[0], p9[1], p9[2]);

	//2
	pFrame->triangle(pTexture5, p10[0], p10[1], p10[2]);
	pFrame->triangle(pTexture5, p11[0], p11[1], p11[2]);
	
	// 点光源
	CMatrix l;
	l.scale(0.7f);
	l.translate(light.x, light.y, light.z);
	pFrame->setWorld(l);
	time += 0.05f;
	pFrame->setLight(light, 1.0f);
	pFrame->transform();
	pFrame->triangle(pLight, light0[0], light0[1], light0[2]);
	pFrame->triangle(pLight, light1[0], light1[1], light1[2]);

	return true;
}
